// @file particle100.vert

// Attributes
attribute vec4 a_position;	// Vertex position							(x, y, z, w)
attribute vec3 a_normal;	// Vertex normal							(x, y, z)
attribute vec2 a_texCoord;	// Vertex texture coordinate				(u, v)


// Uniforms
uniform mat4 u_worldMatrix;
uniform mat4 u_viewMatrix;
uniform mat4 u_projectionMatrix;

//uniform mat4 u_worldViewProjectionMatrix;					// Matrix to transform a position to clip space
//uniform mat4 u_inverseTransposeWorldViewMatrix;				// Matrix to transform a normal to view space

uniform vec4 u_matrixPalette[SKINNING_JOINT_COUNT * 3];		// Array of 4x3 matrices


// Varyings
varying vec2 v_texCoord;

varying vec4 v_0;

#include "skinning-none.vert" 


void main() {
	v_texCoord = a_texCoord;

	vec4 position = a_position;
	float id = position.z;

	vec4 sk = vec4(u_matrixPalette[0].w,
		u_matrixPalette[1].w,
		u_matrixPalette[2].w,
		1);


	float speed = 30.0;
	float spread = 0.25;
	float height = 3.0;
	float size = 7.8;

	float unitY = frac(id * 0.01 + speed * sk.z);

	float loopAng = 20.0 * sk.z * 2.0 * 3.141592;

	float y = height * unitY;
	float env = 1.0;
	if (unitY < 0.1) {
		env = pow(unitY / 0.1, 0.5);
	}
	float rr = spread * env;

	vec4 oPos = vec4(rr*cos(loopAng + id*62.0), y, rr*sin(loopAng + id*163.0), 1.0);
	vec4 wPos = u_worldMatrix * oPos;
	vec4 viewPos = u_viewMatrix * wPos;

// JWnő
	float side = 0.5;

	viewPos.xy += position.xy * side * 0.5;
	vec4 pPos = u_projectionMatrix * viewPos;

	gl_Position = pPos;

	v_0 = u_matrixPalette[0];
}
